package com.foo.jpaweb.model

import javax.persistence.{EntityManager,Persistence}

import scala.collection.jcl.{BufferWrapper,SetWrapper}

import net.liftweb.http.RequestVar
import net.liftweb.util.{Can,Full}

object Model {
  lazy val factory = Persistence.createEntityManagerFactory("jpaweb")

  private def openEM () = {
    val em = factory.createEntityManager()
    em.getTransaction().begin()
    em
  }

  private def closeEM (em : EntityManager) = {
    if (em.getTransaction().getRollbackOnly()) {
      em.getTransaction().rollback()
    } else {
      em.getTransaction().commit()
    }

    em.close()
  }

  object emVar extends RequestVar(openEM()) {
    override def cleanupFunc : Can[() => Unit]= Full(() => closeEM(this.is))
  }

  def em = emVar.is

  // Implicit defs to help with entity member access
  implicit def setToWrapper[A](set : java.util.Set[A]) = new SetWrapper[A]{override def underlying = set}
  implicit def listToWrapper[A](list : java.util.List[A]) = new BufferWrapper[A]{override def underlying = list}
}
